package com.blazinggames.retro.classic.search;
// Java Imports

// Blazing Games Imports
import com.blazinggames.urcl.data.Config;
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.text.*;

/**
 * Ultimate Retro Project Episode 1: Classic Doors
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * Runs the Classic doors game.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class Doors 
extends TextDisplay
{
// *********************
// ***** CONSTANTS *****
// *********************
    
    private static final String NUM_LEVELS = "01NUM_LEVELS";
    private static final String LEVEL="01LEVEL";
    private static final String INSTRUCTIONS="01INSTRUCTIONS";

// *********************
// ***** VARIABLES *****
// *********************
    
    private String _levelNames[];
    private int _levelSizes[];
    private int _levelGuesses[];
    private Instructions _instructions;

// **************************************    
// ***** METHODS - From TextDisplay *****
// **************************************

	/**
	 * Initializes the game
	 * 
	 * @param dh DisplayHandler that manages everything
	 * @return true if initialized successfully
	 */
    public boolean initialize(DisplayHandler dh)
    {
        super.initialize(dh);
        int n = _config.getIntProperty(NUM_LEVELS, 0);
        if (n < 1)
            n = 1;
        _levelNames = new String[n];
        _levelSizes = new int[n];
        _levelGuesses = new int[n];
        String args[];
        for (int cntr = 0; cntr < n; ++cntr)
        {
            args = _config.parseString(_config.getStringProperty(LEVEL+"["+cntr+"]","unnamed,100,7"),',');
            if (args.length != 3)
            {
                _levelNames[cntr] = "Play the game";
                _levelSizes[cntr] = 100;
                _levelGuesses[cntr] = 7;
            }
            else
            {
                _levelNames[cntr] = args[0];
                _levelSizes[cntr] = _config.parseInt(args[1], 100);
                _levelGuesses[cntr] = _config.parseInt(args[2], 7);
            }
        }

        return true;
    }
    
    /**
     * game run loop.
     */
    public void run()
    {
        String s;
        int choice;
        
        printHeader();
        do
        {
            println();
            println("Please choose:");
            println(" 0. Instructions");
            for (int cntr = 0; cntr < _levelNames.length; ++cntr)
            {
                println(" " + (cntr+1) + ". " + _levelNames[cntr]);
            }
            int nQuit = _levelNames.length + 1;
            if (_canQuit)
                println(" " + nQuit + ". Quit");
            print("Type number then return: ");
            s = input();
            choice = _config.parseInt(s, -1);
            println(s);
            println();
            if (choice == 0)
                instructions();
            else if ((choice > 0) && (choice < nQuit))
                game(_levelSizes[choice-1], _levelGuesses[choice-1]);
            else if (choice == nQuit)
            {
                if (_canQuit)
                    _displayHandler.displayEvent(this, "SHOW 1");
                else
                    println("Quit command is unavailable");
            }
            else
            {
                println("You have made an invalid choice!");
                println("Please try again.");
            }
        }
        while(_active);
        _displayHandler.log("DoorsDisplay run thread exiting");
    }
    
    /**
     * Support function for displaying header information.
     */
    private void printHeader()
    {
        println("*** Classic DOORS v 1.10.031110 ***");
        println();
        println("Copyright (c) 2001, 2003");
        println("Blazing Games Inc.");
        println("All Rights Reserved.");
        println();
    }

	/**
	 * support function for displaying instructions
	 */
    private void instructions()
    {
        if (_instructions == null)
        {
            println("Grabbing instructions...");
            _res.addTextGroup(INSTRUCTIONS, _config.getStringProperty(INSTRUCTIONS, "tank.txt"));
            _instructions = new Instructions(_res.getTextGroup(INSTRUCTIONS));
            println("Done.");
            println();
        }
        _instructions.showInstructions(this);
    }
    
    /**
     * Runs a round of the game.
     * 
     * @param rooms number of rooms in the game
     * @param guesses number of guesses the player is allowed
     */
    private void game(int rooms, int guesses)
    {
        String s;
        int friend = rand(rooms);
        int count = 0;
        int n;
        _displayHandler.log("Player at " + friend);
        println ("summary: rooms " + rooms + " guesses " + guesses);
        boolean gameInProgress = true;
        do
        {
            ++count;
            print ("Guess " + count + ": ");
            s = input();
            println(s);
            n = _config.parseInt(s, 0);
            if ((n < 1) || (n > rooms))
            {
                println("*INVALID GUESS*");
                --count;
            }
            else if (n == friend)
            {
                println("Well done! you found your friend!");
                gameInProgress = false;
            }
            else if (count >= guesses)
            {
                println("Sorry!");
                println("That was your last guess!");
                println("Your friend was in room " + friend);
                gameInProgress = false;
            }
            else
            {
                if (n > friend)
                    println("Your guess was too high");
                else
                    println("Your guess was too low");
            }
        }
        while (gameInProgress);
        
    }
}
